@@ -5,6 +5,8 @@ var httpObj;
 var nextKey;
 var nowKey;
 var sessionId;
+var keepConnect;
+var cookieName;
 
 function createXMLHttpRequest() {
   try { return new ActiveXObject("Msxml2.XMLHTTP");    } catch(e) {}
@@ -26,13 +28,11 @@ function sendHello(){
       rand=Math.random();
       nextKey=hex_md5(String(rand));
     }catch(e){
-      document.getElementById("watchdog").innerHTML = appletDescription;
+	showError();
     }
   }
-
-  // java communication
   else{
-    document.getElementById("watchdog").innerHTML = appletDescription;
+      showError();
   }
 }
 
@@ -47,29 +47,48 @@ function displayResponse()
         document.getElementById("dispmsg").innerHTML = hh+':'+mm;
         if(httpObj.responseText=="hello") sendHello();
       }else{
-        document.getElementById("dispmsg").innerHTML = "Error!";
+	  showError();
       }
     }
   }catch(e){
-    document.getElementById("watchdog").innerHTML = appletDescription;
+      showError();
+  }
+}
+
+function showError(){
+      document.getElementById("watchdog").innerHTML = "Error!, Close This page";
+      document.getElementById("watchdog").style.backgroundColor = "pink";
+      document.title = "Useless Page";
+      keepConnect=false;
+}
+
+function clearCookie(){
+  if(keepConnect){
+    document.cookie = cookieName+"=;";
   }
 }
 
 function endHttpkeep(){
+
+  //clear cookie
+  clearCookie(); 
+
   if(httpObj!=null){
     httpObj.abort();
   }
 }
 
-function initHttpkeep(urlArg, appletArg, sid)
+function initHttpkeep(urlArg, sid, ckName)
 {
+
   // save arguments
   httpkeepUrl=urlArg;
-  appletDescription=appletArg;
   nowKey=hex_md5(sid);
+  cookieName=ckName;
   sessionId=sid;
   rand=Math.random();
   nextKey=hex_md5(String(rand));
+  keepConnect=true;
 
   // set timeout and send hello 
   sendHello();
