@@ -65,6 +65,9 @@ int getMacAddrFromArp(char *clientAddr4, char* macAddr4)
   /* cut off string at endp */
   *endp='\0';
 
+  /* return if invalid address */
+  if((strstr(startp, ":"))==NULL) return -1;
+
   /* save to macAddr4 */
   strncpy(macAddr4, startp, ADDRMAXLN);
     
