@@ -53,8 +53,8 @@ int authenticateUser(char *userid, char *passwd)
   int timeout;
 
   /* if no userid or no passwd, then deny */
-  if(userid[0]=='\0')return DENY;
-  if(passwd[0]=='\0')return DENY;
+  //if(userid[0]=='\0')return DENY;
+  //if(passwd[0]=='\0')return DENY;
 
   /* authserver get timeout value */
   timeout=atoi(GetConfValue("AuthServer/Timeout"));
