@@ -33,12 +33,9 @@ Modified by Katsuhiko Eguchi
 #define hex2num(x)  ((x)>='A' ? ((x) & 0XDF) - 'A' +10 : ((x) - '0'))
 
 int isHttpWatchEnableClient(void);
-int isJavaWatchEnableClient(void);
 void split(char content[], char *name[], char *value[], char *next[]);
 void decode(char *string);
 
-char language[WORDMAXLN]; /* message language in java applet */
-
 /*******************************/
 /* get the client addr         */
 /*******************************/
@@ -50,61 +47,88 @@ void getClientAddr(char *clientAddr)
 /********************************************/
 /* get Post data from the client  */
 /********************************************/
-int getPostData(char *userid, char *password, char *clientAddr4, int *durationPtr, int *durationEntered)
+int getPostData(char *userid, char *password, char *clientAddr4, int *durationPtr, int *durationEntered, char *language, char *redirectedUrl)
 {
-  int contentLen;
-  char content[BUFFMAXLN];
+  int contentLen=0;
+  int queryStrLen=0;
+  char content[BUFFMAXLN]="";
+  char queryStr[BUFFMAXLN]="";
   char *name[1];
   char *value[1];
   char *next[1];
   char *ptr;
-  char durationStr[WORDMAXLN];
-  char langList[BUFFMAXLN];
-  char encodeAddr4[ADDRMAXLN];
-  char accessAddr[ADDRMAXLN];
+  char durationStr[WORDMAXLN]="";
+  char langList[BUFFMAXLN]="";
+  char encodeAddr4[ADDRMAXLN]="";
+  char accessAddr[ADDRMAXLN]="";
 
   /* get content sent from web input */
-  if(getenv("CONTENT_LENGTH")==NULL){
-    err_msg("ERR at %s#%d: CONTENT_LENGTH is not defined",__FILE__,__LINE__);
-    return FALSE;
-  }
-
-  contentLen=atoi(getenv("CONTENT_LENGTH"));
-  if(contentLen==0){
-    err_msg("ERR at %s#%d: CONTENT_LENGTH is zero",__FILE__,__LINE__);
-    return FALSE;
-  }
-
-  contentLen++; /* for terminate ch */
-  if(contentLen > BUFFMAXLN) contentLen=BUFFMAXLN;
-  if(fgets(content, contentLen, stdin) == NULL){
-    content[0]='\0';
-  }
-
-  /* get items from string */
-  userid[0]='\0';
-  password[0]='\0';
-  encodeAddr4[0]='\0';
-  language[0]='\0';
-  durationStr[0]='\0';
-
-  ptr=content;
-
-  while(ptr!=NULL){
-    split(ptr, name, value, next);
-
-    if(strstr(name[0], "userid")!=NULL){
-      strncpy(userid, value[0], USERMAXLN);
-    }else if(strstr(name[0], "password")!=NULL){
-      strncpy(password, value[0], PASSMAXLN);
-    }else if(strstr(name[0],"remote_addr")!=NULL){
-      strncpy(encodeAddr4,value[0],ADDRMAXLN);
-    }else if(strstr(name[0], "language")!=NULL){
-      strncpy(language, value[0], WORDMAXLN);
-    }else if(strstr(name[0], "duration")!=NULL){
-      strncpy(durationStr, value[0], WORDMAXLN);
+  if(getenv("CONTENT_LENGTH")!=NULL
+     &&(contentLen=atoi(getenv("CONTENT_LENGTH")))!=0){
+    
+    contentLen++; /* for terminate ch */
+    if(contentLen > BUFFMAXLN) contentLen=BUFFMAXLN;
+    if(fgets(content, contentLen, stdin) == NULL){
+      content[0]='\0';
     }
-    ptr=next[0];
+    
+    /* get items from string */
+    userid[0]='\0';
+    password[0]='\0';
+    encodeAddr4[0]='\0';
+    language[0]='\0';
+    durationStr[0]='\0';
+    redirectedUrl[0]='\0';
+    
+    ptr=content;
+    
+    while(ptr!=NULL){
+      split(ptr, name, value, next);
+      
+      if(strstr(name[0], "userid")!=NULL){
+	strncpy(userid, value[0], USERMAXLN);
+      }else if(strstr(name[0], "password")!=NULL){
+	strncpy(password, value[0], PASSMAXLN);
+      }else if(strstr(name[0],"remote_addr")!=NULL){
+	strncpy(encodeAddr4,value[0],ADDRMAXLN);
+      }else if(strstr(name[0], "language")!=NULL){
+	strncpy(language, value[0], WORDMAXLN);
+      }else if(strstr(name[0], "duration")!=NULL){
+	strncpy(durationStr, value[0], WORDMAXLN);
+      }else if(strstr(name[0], "redirected_url")!=NULL){
+	strncpy(redirectedUrl, value[0], BUFFMAXLN);
+      }
+      ptr=next[0];
+    }
+  }
+
+  /* get paremeters from query string (not post but in url) */
+  if(getenv("QUERY_STRING")!=NULL
+     &&(queryStrLen=strlen(getenv("QUERY_STRING")))!=0){
+
+    /* get html access parameter string */
+    strncpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
+
+    /* split language and address in content
+       [addr=0-0-0&lang=ja&redirectedurl=xxxx] */
+    ptr=queryStr;
+    while(ptr!=NULL){
+      split(ptr, name, value, next);
+      if(strstr(name[0], "addr")!=NULL){
+	strncpy(encodeAddr4,value[0],ADDRMAXLN);
+      }else if(strstr(name[0], "lang")!=NULL){
+	strncpy(language, value[0], WORDMAXLN);
+      }else if(strstr(name[0], "redirectedurl")!=NULL){
+	strncpy(redirectedUrl, value[0], BUFFMAXLN);
+      }
+      ptr=next[0];
+    }
+  }
+
+  /* no content and no query string */
+  if(contentLen==0 && queryStrLen==0){
+    err_msg("ERR at %s#%d: no parameter is aquired",__FILE__,__LINE__);
+    return FALSE;
   }
 
   /* decode the HTTP encoding */
@@ -113,6 +137,7 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
   decode(encodeAddr4);
   decode(language);
   decode(durationStr);
+  decode(redirectedUrl);
 
  /* if not available language, use first lang */
   strncpy(langList, GetConfValue("HtmlLangs"), BUFFMAXLN);  /* list of available languages */
@@ -153,19 +178,105 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
      && strncmp(accessAddr, clientAddr4, ADDRMAXLN)!=0){  /* and not same */
     strncpy(clientAddr4, accessAddr, ADDRMAXLN);
   }
+
+  /* if no userid, set useid="?" */
+  if(isNull(userid)){
+    userid[0]='?';
+    userid[1]='\0';
+  }
+
   return TRUE;
 }
 
 
+/********************************************/
+/* get data related to cookie from client  */
+/********************************************/
+int getCookieData(char *userid, char *clientAddr4, int *duration, int *durationEntered, char *language)
+{
+  char cookie[SIDMAXLN]=""; /* md5 session key from cookie */
+  char useridInCookie[USERMAXLN]=""; /* userid from cookie */
+  char macAddr[ADDRMAXLN]=""; /* mac address from arp */
+  char macAddrInDb[ADDRMAXLN]=""; /* mac address in DB */
+
+  /* get cookie values */
+  if(!GetAuthCookie(cookie,useridInCookie)) return FALSE; 
+
+  /* get related info from DB */ 
+  if(!GetSessionInfoFromDb(cookie, userid, clientAddr4, macAddrInDb, 
+			   duration, durationEntered, language)) return FALSE;
+    
+  /* if userid is changed, cookie auth is failed */
+  if(strcmp(useridInCookie, userid)!=0) return FALSE;
+
+  /* if mac addr is changed, cookie auth is failed */
+  GetMacAddrFromArp(clientAddr4, macAddr);
+  if(strcmp(macAddr, macAddrInDb)!=0) return FALSE;
+
+  return TRUE;
+}
+
+/*********************************************/
+/* get HTTP-Cookie for OpengateAuth          */
+/*********************************************/
+  /* cookie string examples 
+  "OpengateAuth=de..ac1&Userid=user1"
+  "OpengateAuth=de..ac1&Userid=user1; xxx=..; yyy=.."
+  "xxx=..; yyy=..; OpengateAuth=de..ac1&Userid=user1"
+  */
+int getAuthCookie(char *cookie, char *userid){
+  char content[BUFFMAXLN];
+  char *name[1];
+  char *value[1];
+  char *next[1];
+  char *ptr=NULL;
+  char *ptrNext=NULL;
+
+  /* reset buffer */
+  cookie[0]='\0';
+  userid[0]='\0';
+
+ /* if exist cookie, copy it to work area */
+  if(isNull(getenv("HTTP_COOKIE"))) return FALSE;
+  strncpy(content, getenv("HTTP_COOKIE"), BUFFMAXLN);
+  ptr=content;
+
+  /* search 'OpengateAuth' cookie string (terminated by ; or \0) */
+  while(ptr!=NULL){
+    if((ptrNext=strstr(ptr, "; "))==NULL) break;          /* search "; " */
+    *ptrNext='\0';                               /* overwrite string end */
+    ptrNext++;                                 /* pointer to next string */
+    while(!isNull(ptrNext)&&*ptrNext==' ') ptrNext++;     /* skip spaces */
+    if(strstr(ptr, COOKIENAME)==ptr) break;          /* exit at matching */
+    ptr=ptrNext;                                    /* check next string */
+  }
+
+  /* get valuses of cookie from "OpengateAuth=de..ac1&Userid=user1" */
+  while(ptr!=NULL){
+    split(ptr, name, value, next);
+
+    if(strstr(name[0], COOKIENAME)!=NULL){
+      strncpy(cookie, value[0], SIDMAXLN);
+    }else if(strstr(name[0], "Userid")!=NULL){
+      strncpy(userid, value[0], USERMAXLN);
+    }
+    ptr=next[0];
+  }
+  
+  if(isNull(cookie)) return FALSE;
+  else return TRUE;
+}
+
 /*********************************************/
 /* deny message to the client            */
 /*********************************************/
-void putClientDeny(char *clientAddr4)
+void putClientDeny(char *clientAddr4, char *language)
 {
-  char denydoc[BUFFMAXLN];
-  char authCgiUrl[BUFFMAXLN];
-  char encodeAddr[ADDRMAXLN];
-  char opengateDir[BUFFMAXLN];
+  char denydoc[BUFFMAXLN]="";
+  char authCgiUrl[BUFFMAXLN]="";
+  char encodeAddr[ADDRMAXLN]="";
+  char opengateDir[BUFFMAXLN]="";
+  char protocol[WORDMAXLN]="";
 
   /* keyword pairs */
   /*  the left key is replaced by the right value */
@@ -174,6 +285,8 @@ void putClientDeny(char *clientAddr4)
       {"%%OPENGATEDIR%%", opengateDir},
       {"%%AUTHCGIURL%%", authCgiUrl},
       {"%%ADDR4%%", encodeAddr},
+      {"%%LANGUAGE%%", language},
+      {"%%PROTOCOL%%", protocol},
       {"",""}  /* DON'T REMOVE THIS LINE */
     };
  
@@ -192,17 +305,19 @@ void putClientDeny(char *clientAddr4)
     encodeAddr[0]='\0';
   }
 
-  /* make path to the denydoc for ssl or non-ssl */
+  /* protocol is ssl or non-ssl */
   if(strcmp(getenv("SERVER_PORT"),GetServicePortStr("https"))==0){
-    snprintf(denydoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	    GetConfValue("OpengateDir"),language,GetConfValue("DenyDocSsl"));
+    strncpy(protocol, "https",WORDMAXLN);
   }else{
-    snprintf(denydoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	    GetConfValue("OpengateDir"),language,GetConfValue("DenyDoc"));
+    strncpy(protocol, "http",WORDMAXLN);
   }
 
+  /* document path to deny doc */
+  snprintf(denydoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
+	    GetConfValue("OpengateDir"),language,GetConfValue("DenyDoc"));
+
   /* replace keyword and send out the file */
-  printf("Content-type: text/html\r\n\r\n\r\n");
+  printf("Content-type: text/html\r\n\r\n");
   HtmlTemplate(denydoc, keys);
 
   return;
@@ -210,7 +325,7 @@ void putClientDeny(char *clientAddr4)
 /*********************************************/
 /* deny message to the client            */
 /*********************************************/
-void putClientRetry(char *lang)
+void putClientRetry(char *language)
 {
   char retrydoc[BUFFMAXLN];
   char externalUrl[BUFFMAXLN];
@@ -224,6 +339,7 @@ void putClientRetry(char *lang)
       {"%%OPENGATEDIR%%", opengateDir},
       {"%%EXTERNALURL%%", externalUrl},
       {"%%AUTHCGIURL%%", authCgiUrl},
+      {"%%LANGUAGE%%", language},
       {"",""}  /* DON'T REMOVE THIS LINE */
     };
 
@@ -242,7 +358,7 @@ void putClientRetry(char *lang)
 
   /* make read in path to the retry document */
   snprintf(retrydoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	  opengateDir,lang,GetConfValue("RetryDoc"));
+	  opengateDir,language,GetConfValue("RetryDoc"));
 
   /* replace keyword and send out the file */
   printf("Content-type: text/html\r\n\r\n\r\n");
@@ -256,7 +372,7 @@ void putClientRetry(char *lang)
 /*********************************************/
 void putClientMsg(char *message)
 {
-  printf("Content-type: text/html\r\n\r\n\r\n");
+  printf("Content-type: text/html\r\n\r\n");
   printf("<HTML><HEAD><TITLE>OpengateMsg</TITLE></HEAD> \r\n");
   printf("<BODY>         \r\n");
   printf("%s\r\n",     message);
@@ -264,25 +380,27 @@ void putClientMsg(char *message)
 
 }
 
+
 /*********************************************/
-/* put accept message and java to the client */
+/* put accept message and javascript to the client */
+/*  httpkeep page is sent in watch-client.c */
 /*********************************************/
-void putClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered)
+void putClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered, char *language, char *cookie, int cookieAuth, char *redirectedUrl)
 {
+
   FILE *fp;
   char buff[BUFFMAXLN];
-  char acceptdoc[BUFFMAXLN];
-  char acceptdoc2url[BUFFMAXLN];
-  char terminateurl[BUFFMAXLN];
+  char acceptDocPath[BUFFMAXLN];
+  char acceptDoc2Url[BUFFMAXLN];
+  char terminateUrl[BUFFMAXLN];
   char httpkeepUrl[BUFFMAXLN];
   char portStr[WORDMAXLN];
   char durationStr[WORDMAXLN];
-  char *acceptDoc;
-
-  char *startPageUrl=GetConfValue("StartPage/Url");
-  int startPageType=atoi(GetConfValue("StartPage/Type"));
+  char *startPageUrl="";
+  char *pAcceptDoc;
   char *opengateDir=GetConfValue("OpengateDir");
   char *opengateServerName=GetConfValue("OpengateServerName");
+  int startPageType=atoi(GetConfValue("StartPage/Type"));
 
   /* select proper accept doc */
   switch(toupper(*GetConfValue("WatchMode"))){
@@ -291,45 +409,32 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
   case 'H':
   default:
     if(isHttpWatchEnableClient()){
-      acceptDoc=GetConfValue("AcceptDocHttp");
-    }else if(isJavaWatchEnableClient()){
-      acceptDoc=GetConfValue("AcceptDocJava");
+      pAcceptDoc=GetConfValue("AcceptDocHttp");
     }else{
-      acceptDoc=GetConfValue("AcceptDocTime");
-    }
-    break;
-
-    /* JAVA watch mode */
-  case 'J':
-    if(isJavaWatchEnableClient()){
-      acceptDoc=GetConfValue("AcceptDocJava");
-    }else if(isHttpWatchEnableClient()){
-      acceptDoc=GetConfValue("AcceptDocHttp");
-    }else{
-      acceptDoc=GetConfValue("AcceptDocTime");
+      pAcceptDoc=GetConfValue("AcceptDocTime");
     }
     break;
 
     /* TIMEOUT watch mode */
   case 'T':
-    acceptDoc=GetConfValue("AcceptDocTime");
+    pAcceptDoc=GetConfValue("AcceptDocTime");
     break;
   }
 
   /* if positive value is set in duration, TIME watch mode is selected */
-  if(durationEntered) acceptDoc=GetConfValue("AcceptDocTime");
+  if(durationEntered) pAcceptDoc=GetConfValue("AcceptDocTime");
 
   /* create path to acceptdoc */
-  snprintf(acceptdoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	 GetConfValue("OpengateDir"),language,acceptDoc);
+  snprintf(acceptDocPath, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
+	 GetConfValue("OpengateDir"),language,pAcceptDoc);
 
 
-  snprintf(acceptdoc2url, BUFFMAXLN, 
+  snprintf(acceptDoc2Url, BUFFMAXLN, 
 	  "http://%s%s/%s/%s",GetConfValue("OpengateServerName"),
 	  GetConfValue("OpengateDir"),language,GetConfValue("AcceptDoc2"));
 
   /* create terminate url [http://<servaddr>:<port>/terminate-<pid>] */
-  snprintf(terminateurl, BUFFMAXLN, "http://%s:%d/terminate-%d", 
+  snprintf(terminateUrl, BUFFMAXLN, "http://%s:%d/terminate-%d", 
 	  GetConfValue("OpengateServerName"), port, getpid());
 
   /* create httpkeep page url
@@ -345,21 +450,37 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
   snprintf(durationStr, WORDMAXLN, "%d", duration/60);
 
   /* open acceptdoc */
-  if((fp=fopen(acceptdoc, "r"))==NULL){
-    err_msg("ERR at %s#%d: cannot open %s",__FILE__,__LINE__,acceptdoc);
+  if((fp=fopen(acceptDocPath, "r"))==NULL){
+    err_msg("ERR at %s#%d: cannot open %s",__FILE__,__LINE__,acceptDocPath);
     PutClientMsg("Cannot find html document");
     return;
   }
 
-  /* read html document from file and send to web */
-  printf("Content-type: text/html\r\n\r\n\r\n");
-  while(fgets(buff, BUFFMAXLN, fp)!=NULL){
+  /* if redirect page is not set, use other setting */
+  if(isNull(redirectedUrl)){
+    if(isNull(GetConfValue("StartPage/Url"))) redirectedUrl=acceptDoc2Url;
+    else  redirectedUrl=GetConfValue("StartPage/Url");
+  }
 
+  /* create start page url to put information */
+  if(cookieAuth==1) startPageUrl=redirectedUrl;
+  else if(startPageType==0) startPageUrl=acceptDoc2Url;
+  else if(startPageType==1) startPageUrl=GetConfValue("StartPage/Url");
+  else if(startPageType==2) startPageUrl=redirectedUrl;
+  else startPageUrl=acceptDoc2Url;
+
+  /* write out html headers */
+  printf("Content-type: text/html\r\n");
+  printf("Set-Cookie: %s=%s&Userid=%s;path=/;\r\n\r\n", COOKIENAME, cookie, userid);
+  
+  /* read html document from file and send to web */
+  while(fgets(buff, BUFFMAXLN, fp)!=NULL){
+    
     /* length check */
     if(strlen(buff)>=BUFFMAXLN-1){
-      err_msg("ERR at %s#%d: too long line in %s",__FILE__,__LINE__,acceptdoc);
+      err_msg("ERR at %s#%d: too long line in %s",__FILE__,__LINE__,acceptDocPath);
     }
-
+    
     /* replace mark */
     htmlReplace(buff, "%%OPENGATESERVERNAME%%", opengateServerName);
     htmlReplace(buff, "%%OPENGATEDIR%%", opengateDir);
@@ -368,20 +489,14 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
     htmlReplace(buff, "%%USERID%%", userid);
     htmlReplace(buff, "%%SESSIONID%%", sessionId);
     htmlReplace(buff, "%%LANGUAGE%%", language);
-    
-    htmlReplace(buff, "%%TERMINATEURL%%", terminateurl);
+    htmlReplace(buff, "%%TERMINATEURL%%", terminateUrl);
     htmlReplace(buff, "%%HTTPKEEPURL%%", httpkeepUrl);
-    
-    /* replace information url mark */
-    if( startPageType==1 ){
-      htmlReplace(buff, "%%STARTURL%%", startPageUrl);
-    }else{
-      htmlReplace(buff, "%%STARTURL%%", acceptdoc2url);
-    }
+    htmlReplace(buff, "%%STARTURL%%", startPageUrl);
     
     /* write out */
     printf("%s",buff);
   }
+  
   /* write end */
   fputs("\r\n\r\n",stdout);
   fclose(fp);
@@ -404,18 +519,6 @@ int isHttpWatchEnableClient(void)
   return TRUE;
 }
 
-/********************************************/
-/* is the client enable to load Java Applet */
-/********************************************/
-int isJavaWatchEnableClient(void)
-{
-  /* some user agent does not support Java Applet */
-  if(RegExMatch(getenv("HTTP_USER_AGENT"),
-		 GetConfValue("JavaWatch/SkipAgentPattern"))) return FALSE;
-
-  return TRUE;
-}
-
 /************************************/
 /* split value for indicated name   */
 /* in content  "name=value&..."     */
@@ -502,27 +605,27 @@ void GetClientAddr(char *clientAddr)
 }
 
 
-int GetPostData(char *userid, char *password, char *clientAddr4, int *durationPtr, int *durationEntered)
+int GetPostData(char *userid, char *password, char *clientAddr4, int *durationPtr, int *durationEntered, char *language, char *redirectedUrl)
 {
   int ret;
 
   if(debug>1) err_msg("DEBUG:=>getPostData( )");
-  ret=getPostData(userid,password,clientAddr4,durationPtr,durationEntered);
-  if(debug>1) err_msg("DEBUG:%d<=getPostData(%s,passwd,%s,%d,%d)",ret,userid,clientAddr4,*durationPtr,durationEntered);
+  ret=getPostData(userid,password,clientAddr4,durationPtr,durationEntered,language,redirectedUrl);
+  if(debug>1) err_msg("DEBUG:%d<=getPostData(%s,%s,%d,%d,%s)",ret,userid,clientAddr4,*durationPtr,durationEntered,language,redirectedUrl);
   return ret;
 }
 
-void PutClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered)
+void PutClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered, char *language, char *cookie, int cookieAuth, char *redirectedUrl)
 {
-  if(debug>1) err_msg("DEBUG:=>putClientAccept(%s,%s,%d,%d,%s,%s,%d,%d,%d)",userid,sessionId,port,pid,clientAddr4,clientAddr6,ipStatus, duration, durationEntered);
-  putClientAccept(userid,sessionId,port,pid,clientAddr4,clientAddr6,ipStatus,duration, durationEntered);
+  if(debug>1) err_msg("DEBUG:=>putClientAccept(%s,%s,%d,%d,%s,%s,%d,%d,%d,%s,%s,%d,%s)",userid,sessionId,port,pid,clientAddr4,clientAddr6,ipStatus, duration, durationEntered, language, cookie, cookieAuth, redirectedUrl);
+  putClientAccept(userid,sessionId,port,pid,clientAddr4,clientAddr6,ipStatus,duration, durationEntered, language, cookie, cookieAuth,redirectedUrl);
   if(debug>1) err_msg("DEBUG:<=putClientAccept( )");
 }
 
-void PutClientDeny(char *clientAddr4)
+void PutClientDeny(char *clientAddr4, char *language)
 {
-  if(debug>1) err_msg("DEBUG:=>putClientDeny(&s)",clientAddr4);
-  putClientDeny(clientAddr4);
+  if(debug>1) err_msg("DEBUG:=>putClientDeny(%s,%s)",clientAddr4,language);
+  putClientDeny(clientAddr4,language);
   if(debug>1) err_msg("DEBUG:<=putClientDeny( )");
 }
 
@@ -548,3 +651,22 @@ int CheckReferer(void)
   if(debug>1) err_msg("DEBUG:(%d)<=checkReferer( )",ret);
   return ret;
 }
+
+int GetAuthCookie(char *cookie, char *userid){
+  int ret;
+
+  if(debug>1) err_msg("DEBUG:=>getAuthCookie( )");
+  ret=getAuthCookie(cookie, userid);
+  if(debug>1) err_msg("DEBUG:%d<=getAuthCookie(%s,%s)",
+		      ret,cookie,userid);
+  return ret;
+}
+
+int GetCookieData(char *userid, char *clientAddr4, int *duration, int *durationEntered, char *language){
+  int ret;
+
+  if(debug>1) err_msg("DEBUG:=>getCookieData( )");
+  ret=getCookieData(userid,clientAddr4,duration,durationEntered,language);
+  if(debug>1) err_msg("DEBUG:%d<=getCookieData(%s,passwd,%s,%d,%d,%s)",ret,userid,clientAddr4,*duration,*durationEntered,language);
+  return ret;
+}
