@@ -1,6 +1,6 @@
 /**************************************************
 opengate server
- module for Controling ip6fw 
+ module for Controling ipfw for IPv6 address 
 
 Copyright (C) 2005 Opengate Project Team
 Written by Katsuhiko Eguchi, 2005 
@@ -24,7 +24,7 @@ Email: watanaby@is.saga-u.ac.jp
 
 #include "opengatesrv.h"
 
-char ruleNumber6[WORDMAXLN];  /* ip6fw rule number in string form */
+char ruleNumber6[WORDMAXLN];  /* ipfw rule number in string form */
 
 int getRuleNumber6(char *clientAddr6);
 int GetRuleNumber6(char *clientAddr6);
@@ -42,7 +42,7 @@ int openClientGate6(char *clientAddr6, char *userid, char *macAddr6, char *userP
 
   Sigfunc *defaultSigFunc;
 
-  /* exclusive exec of ip6fw to avoid overlapped rule number */
+  /* exclusive exec of ipfw to avoid overlapped rule number */
 
   /**** prepare ****/
   /* open lockfile */
@@ -75,13 +75,14 @@ int openClientGate6(char *clientAddr6, char *userid, char *macAddr6, char *userP
   }
 
   /**** write rules ****/
-  if(atoi(GetConfValue("Ip6fwScript/Enable"))){
+  if(atoi(GetConfValue("IpfwScript/Enable"))){
     /********** use perl script to control firewall ************/
 
-    if(Systeml(1, GetConfValue("Ip6fwScript/Path"),GetConfValue("Ip6fwPath"),
+    if(Systeml(1, GetConfValue("IpfwScript/Path"),GetConfValue("IpfwPath"),
 	       ruleNumber6,clientAddr6,
-	       userid,macAddr6,userProperty,(char *)0) != 0){
-      err_msg("ERR at %s#%d: exec ip6fw script error",__FILE__,__LINE__);
+	       userid,macAddr6,userProperty,
+	       GetConfValue("IpfwTagNumber"),(char *)0) != 0){
+      err_msg("ERR at %s#%d: exec ipfw script error",__FILE__,__LINE__);
       ret=1; /* abmormal */
     }
 
@@ -91,21 +92,23 @@ int openClientGate6(char *clientAddr6, char *userid, char *macAddr6, char *userP
 
   }else{
     /********** direct control of firewall **********************/
-    /********** add outgoing ip6fw rule for the client *************/
-    if(Systeml(1, GetConfValue("Ip6fwPath"),"-q","add",ruleNumber6,"allow","all",
-	       "from",clientAddr6,"to","any",(char *)0) != 0){
-	      err_msg("ERR at %s#%d: exec ip6fw add error",__FILE__,__LINE__);
+    /********** add outgoing ipfw rule for the client *************/
+    if(Systeml(1, GetConfValue("IpfwPath"),"-q","add",ruleNumber6,
+	       "count","tag",GetConfValue("IpfwTagNumber"),
+	       "ip","from",clientAddr6,"to","any",(char *)0) != 0){
+	      err_msg("ERR at %s#%d: exec ipfw add error",__FILE__,__LINE__);
 	      ret=1;
     }
     
     /* lock is not necessary in following exec */
     Unlock(fd);
     Close(fd);    /* because reserved number is used */
-    
-    /********** add incoming ip6fw rule for the client *************/
-    if(Systeml(1, GetConfValue("Ip6fwPath"),"-q","add",ruleNumber6,"allow","all",
-	       "from","any","to",clientAddr6,(char *)0) != 0){
-	      err_msg("ERR at %s#%d: exec ip6fw add error",__FILE__,__LINE__);
+
+    /********** add incoming ipfw rule for the client *************/
+    if(Systeml(1, GetConfValue("IpfwPath"),"-q","add",ruleNumber6,
+	       "count","tag",GetConfValue("IpfwTagNumber"),
+	       "ip","from","any","to",clientAddr6,(char *)0) != 0){
+	      err_msg("ERR at %s#%d: exec ipfw add error",__FILE__,__LINE__);
 	      ret=1; /* abnormal */
     }
   }
@@ -122,7 +125,7 @@ void closeClientGate6(struct clientAddr *pClientAddr, char *userid, char *macAdd
   int hour, min, sec;
   time_t timeOut;
   
-  /********** del ip6fw rule for the client *************/
+  /********** del ipfw rule for the client *************/
   DelIp6fwRule(pClientAddr->ruleNumber);
   
   timeOut = time(NULL);
@@ -138,27 +141,23 @@ void closeClientGate6(struct clientAddr *pClientAddr, char *userid, char *macAdd
 
 
 /***********************************************/
-/* delete ip6fw rule                            */
+/* delete ipfw rule                            */
 /***********************************************/
 void delIp6fwRule(char *ruleNumber)
 {
   int ruleCount;
-  int i;
 
   /* get rule count */
   ruleCount = CountRuleNumber6(ruleNumber);
 
   /* delete rule */
-  /* [ip6fw del num] deletes one rule registered in the rule number */
-  for(i=0;i<ruleCount;i++){
-    if(Systeml(1, GetConfValue("Ip6fwPath"),"del",ruleNumber,(char *)0) != 0){
-      err_msg("ERR at %s#%d: exec ip6fw del error",__FILE__,__LINE__);
-    }
+  if(Systeml(1, GetConfValue("IpfwPath"),"delete",ruleNumber,(char *)0) != 0){
+    err_msg("ERR at %s#%d: exec ipfw del error",__FILE__,__LINE__);
   }
 }
   
-  /**************************************/
-/* get unused ip6fw rule number        */
+/**************************************/
+/* get unused ipfw rule number        */
 /* error if addr is already in rules */ 
 /* return value ret>0: acquired rule number that can be used */
 /*              ret=-1: no rule number available */
@@ -178,8 +177,8 @@ int getRuleNumber6(char *clientAddr6)
   int fileStatus;
   enum status {NORMAL, ABNORMAL, FOUND, NOTFOUND, DUPLICATED};
 
-  if((fpipe=Popenl(1, "r", GetConfValue("Ip6fwPath"),"list",(char *)0)) == NULL){ 
-      err_msg("ERR at %s#%d: exec ip6fw list error",__FILE__,__LINE__);
+  if((fpipe=Popenl(1, "r", GetConfValue("IpfwPath"),"list",(char *)0)) == NULL){ 
+      err_msg("ERR at %s#%d: exec ipfw list error",__FILE__,__LINE__);
   }
   
   /* search unused rule number in the list read from pipe */
@@ -191,9 +190,9 @@ int getRuleNumber6(char *clientAddr6)
   fileStatus=NORMAL;
 
   /* get rule range from config */
-  ip6fwmin=atoi(GetConfValue("Ip6fwRule/Min"));
-  ip6fwmax=atoi(GetConfValue("Ip6fwRule/Max"));
-  ip6fwinterval=atoi(GetConfValue("Ip6fwRule/Interval"));
+  ip6fwmin=atoi(GetConfValue("IpfwRule/Min"));
+  ip6fwmax=atoi(GetConfValue("IpfwRule/Max"));
+  ip6fwinterval=atoi(GetConfValue("IpfwRule/Interval"));
 
   /* each port is checked whether it can be used for new rule or not */
   for(num=ip6fwmin;num<=ip6fwmax;num+=ip6fwinterval){
@@ -206,7 +205,7 @@ int getRuleNumber6(char *clientAddr6)
 	break;
       }
       if( sscanf(buf, "%d", &readinNum) !=1 ){
-	err_msg("ERR at %s#%d: abnormal ip6fw response[ %s ]",
+	err_msg("ERR at %s#%d: abnormal ipfw response[ %s ]",
 		__FILE__,__LINE__,buf);
 	fileStatus=ABNORMAL; /* abnormal responsem exit internal loop */
 	break;
@@ -257,11 +256,11 @@ int getRuleNumber6(char *clientAddr6)
   Pclose(fpipe);
 
   if(fileStatus==ABNORMAL){
-    err_msg("ERR at %s#%d: abnormal ip6fw response ",__FILE__,__LINE__);
+    err_msg("ERR at %s#%d: abnormal ipfw response ",__FILE__,__LINE__);
     return -2;
   }
   if(portStatus==NOTFOUND){
-    err_msg("ERR at %s#%d: cannot get unused ip6fw number",__FILE__,__LINE__);
+    err_msg("ERR at %s#%d: cannot get unused ipfw number",__FILE__,__LINE__);
     return -1;
   }
   if(portStatus==DUPLICATED){
@@ -274,7 +273,7 @@ int getRuleNumber6(char *clientAddr6)
 }
 
 /*******************************/
-/* get packet count from ip6fw  */
+/* get packet count from ipfw  */
 /*******************************/
 int getPacketCount6(char *ruleNumber)
 {
@@ -284,8 +283,8 @@ int getPacketCount6(char *ruleNumber)
   int packets,packetsSum;
 
   /* exec proc */
-  if((fpipe=Popenl(1, "r", GetConfValue("Ip6fwPath"),"-a","list",ruleNumber,(char *)0)) == NULL){ 
-    err_msg("ERR at %s#%d: exec ip6fw -a list error",__FILE__,__LINE__);
+  if((fpipe=Popenl(1, "r", GetConfValue("IpfwPath"),"-a","list",ruleNumber,(char *)0)) == NULL){ 
+    err_msg("ERR at %s#%d: exec ipfw -a list error",__FILE__,__LINE__);
     return 0; /* abnormal */
   }
   
@@ -313,8 +312,8 @@ int countRuleNumber6(char *ruleNumber)
   int ruleCount;
 
   /* exec proc */
-  if((fpipe=Popenl(1, "r", GetConfValue("Ip6fwPath"),"list",ruleNumber,(char *)0)) == NULL){ 
-    err_msg("ERR at %s#%d: exec ip6fw list error",__FILE__,__LINE__);
+  if((fpipe=Popenl(1, "r", GetConfValue("IpfwPath"),"list",ruleNumber,(char *)0)) == NULL){ 
+    err_msg("ERR at %s#%d: exec ipfw list error",__FILE__,__LINE__);
   }
   
   /* count line read from pipe */
