@@ -82,7 +82,8 @@ int openClientGate4(char *clientAddr4, char *userid, char *macAddr4, char *userP
 
     if(Systeml(1, GetConfValue("IpfwScript/Path"),GetConfValue("IpfwPath"),
 	       ruleNumber4,clientAddr4,
-	       userid,macAddr4,userProperty,(char *)0) != 0){
+	       userid,macAddr4,userProperty,
+	       GetConfValue("IpfwTagNumber"),(char *)0) != 0){
       err_msg("ERR at %s#%d: exec script error",__FILE__,__LINE__);
       ret=1;  /* abnormal */
     }
@@ -94,8 +95,9 @@ int openClientGate4(char *clientAddr4, char *userid, char *macAddr4, char *userP
   }else{
     /********** direct control of firewall **********************/
     /********** add outgoing ipfw rule for the client *************/
-    if(Systeml(1, GetConfValue("IpfwPath"),"-q","add",ruleNumber4,"allow","ip",
-	       "from",clientAddr4,"to","any",(char *)0) != 0){
+    if(Systeml(1, GetConfValue("IpfwPath"),"-q","add",ruleNumber4,
+	       "count","tag",GetConfValue("IpfwTagNumber"),
+	       "ip","from",clientAddr4,"to","any",(char *)0) != 0){
       err_msg("ERR at %s#%d: exec ipfw add error",__FILE__,__LINE__);
       ret=1;  /* abnormal */
     }
@@ -104,8 +106,9 @@ int openClientGate4(char *clientAddr4, char *userid, char *macAddr4, char *userP
     Unlock(fd);
     Close(fd);    /* because reserved number is used */
     
-    if(Systeml(1, GetConfValue("IpfwPath"),"-q","add",ruleNumber4,"allow","ip",
-	       "from","any","to",clientAddr4,(char *)0) != 0){
+    if(Systeml(1, GetConfValue("IpfwPath"),"-q","add",ruleNumber4,
+	       "count","tag",GetConfValue("IpfwTagNumber"),
+	       "ip","from","any","to",clientAddr4,(char *)0) != 0){
       err_msg("ERR at %s#%d: exec ipfw add error",__FILE__,__LINE__);
       ret=1; /* abnormal */
     }
@@ -149,7 +152,7 @@ void delIpfwRule(char *ruleNumber){
   if(count>0){
     /* exec ipfw del */
     /* [ipfw del rule] deletes all rule of the rule number at one call */
-    if(Systeml(1, GetConfValue("IpfwPath"),"del",ruleNumber,(char *)0) != 0){
+    if(Systeml(1, GetConfValue("IpfwPath"),"delete",ruleNumber,(char *)0) != 0){
       err_msg("ERR at %s#%d: exec ipfw del error",__FILE__,__LINE__);
     }
   }
