@@ -183,6 +183,10 @@ int getMacAddrFromNdp(char *clientAddr6, char* macAddr6)
     if(strstr(buf,clientAddr6)==NULL)continue;
     startp = strtok(buf," "); /* first token */
     startp = strtok(NULL," "); /* second token */
+
+    /* skip if invalid address */
+    if(strstr(startp, ":")==NULL)continue;
+
     strcpy(macAddr6,startp);
   }
 
