@@ -26,7 +26,7 @@ Email: watanaby@is.saga-u.ac.jp
 
 extern char ruleNumber4[WORDMAXLN];  /* ipfw rule number in string form  */
 extern char ruleNumber6[WORDMAXLN];  /* ip6fw rule number in string form */
-extern char language[WORDMAXLN];
+char language[WORDMAXLN];
 
 char clientAddr4[ADDRMAXLN]="";  /* client addr (nnn.nnn.nnn.nnn) */
 char clientAddr6[ADDRMAXLN]="";  /* client addr (nnnn:nnnn:xxxx::xxxx) 128bit */
@@ -42,10 +42,14 @@ char userProperty[BUFFMAXLN];
 time_t timeIn, timeOut;
 int ipStatus;              /* flag for IPV4ONLY,IPV6ONLY or IPV46DUAL */
 int connectionMode;   /* client connect mode */
+char *mode[3]={"NONE","HTTP","TIME"};
+
 /* variable to measuring processing time (in msec) */
 struct timeval timeBeginCgi, timeBeginWait, timeConnect, timeDisconnect;
 
-char sessionId[BUFFMAXLN];    /* session ID */
+char sessionId[SIDMAXLN]="";    /* session ID */
+char cookie[SIDMAXLN]="";    /* cookie */
+char redirectedUrl[BUFFMAXLN]="";  /* redirected URL (requested URL before fwd) */
 
 void PutCloseMsg(time_t timeOut, time_t timeIn);
 void SetProcessTitle(char *useridshort, char *clientAddr4, char * ruleNumber4, char *clientAddr6, char * ruleNumber6, int ipStatus);
@@ -57,15 +61,16 @@ void closeExit(int signo);
 /***************************************************/
 int  main(int argc, char **argv)
 {
-  char password[PASSMAXLN];
+  char password[PASSMAXLN]="";
   int port;
   int dummyfd[2];
   int pid;
   int parentpid;
-  int duration;     /* requested usage duration */
-  int durationEntered; /* the duration value is entered or not */
+  int duration=0;     /* requested usage duration */
+  int durationEntered=0; /* the duration value is entered or not */
   int authResult=DENY;
   int authNum=1; /* present authserver number to check user */
+  int cookieAuth=FALSE; /* Auth with HTTP-Cookie is passed */
 
   /* drop root privilege */
   seteuid(getuid());
@@ -86,18 +91,26 @@ int  main(int argc, char **argv)
   /* get default language at the top of lang list */
   sscanf(GetConfValue("HtmlLangs"), "%s", language);
 
- /* check referer */
+  /* check referer */
   if(CheckReferer()==FALSE){
     PutClientRetry(language);
     return 0;
   }
 
-  /* get POST data */
-  if(GetPostData(userid, password, clientAddr4, &duration, &durationEntered)==FALSE){
+  /* get POST data from HTTP request */
+  if(GetPostData(userid, password, clientAddr4, 
+		 &duration, &durationEntered, language, redirectedUrl)==FALSE){
     PutClientRetry(language);
     return 0;
   }
 
+  /* try to authentication with cookie data, */
+  /* if cookie auth is enabled. */
+  if( (*GetConfValue("EnableCookieAuth")!='0') ){
+    cookieAuth=GetCookieData(userid, clientAddr4,
+			   &duration,&durationEntered,language);
+  }
+
   /* split user@server to user and server */
   SplitId(userid, useridshort, extraId);
 
@@ -109,32 +122,34 @@ int  main(int argc, char **argv)
 
   /* check enable IP versions */
   if((ipStatus=CheckIpVersions(clientAddr4, clientAddr6))==0){
-      PutClientMsg("Error: Please contact to the administrator");
+      PutClientMsg("Error: Please contact the administrator(1)");
       return 0;
   }
 
   /* get MAC address from arp and ndp */
   GetMacAddr(clientAddr4, macAddr4, clientAddr6, macAddr6,ipStatus);
 
-
   /* check user by authenticate servers */
-  while(SelectNextAuthServer()){
+  if(cookieAuth) authResult=ACCEPT;
+  else{
+    while(SelectNextAuthServer()){
 
-    /* authenticate the user with authentication servers */
-    authResult=AuthenticateUser(useridshort, password);
+      /* authenticate the user with authentication servers */
+      authResult=AuthenticateUser(useridshort, password);
 
-    /* if pass, exit the loop */
-    if(authResult==ACCEPT) break;
-
-    /* else continue the loop */
-    err_msg("DENY:(auth%d) user %s from %s at %s", 
-	    authNum, userid, clientAddr4, macAddr4);
-    authNum++;
+      /* if success, exit loop */
+      if(authResult==ACCEPT) break;
+	 
+      /* else put out error message */
+      err_msg("DENY:(auth%d) user %s from %s at %s", 
+	      authNum, userid, clientAddr4, macAddr4);
+      authNum++;
+    }
   }
 
   /* if not authenticate, send deny to the client and exit */
   if(authResult!=ACCEPT){
-    PutClientDeny(clientAddr4);
+    PutClientDeny(clientAddr4, language);
     return 0;
   }
 
@@ -149,12 +164,13 @@ int  main(int argc, char **argv)
     return 0;
   }
 
-  /* create session ID */
+  /* create sessionID(for hello check) and HTTP-Cookie(for auth) */
   CreateSessionId(sessionId);
+  CreateCookie(cookie);
 
   /* set terminate signal handler */
   if(Signal(SIGTERM, closeExit)==SIG_ERR){
-    PutClientMsg("Error: Please contact to the administrator");    
+    PutClientMsg("Error: Please contact the administrator(2)");    
     return 0;
   }
 
@@ -164,13 +180,13 @@ int  main(int argc, char **argv)
   timeIn=time(NULL);
 
   /* set (ruleNumber,userid,clientAddr) in process title */
-  SetProcessTitle(useridshort,clientAddr4,ruleNumber4,clientAddr6,ruleNumber6,ipStatus);
+  SetProcessTitle(userid,clientAddr4,ruleNumber4,clientAddr6,ruleNumber6,ipStatus);
   
   /* get temporary port for server-listen */
   port=GetListenPort();
   if(port<0){
     err_msg("ERR at %s#%d: cannot get unused listen port",__FILE__,__LINE__);
-    PutClientMsg("Error: Please contact to the administrator");
+    PutClientMsg("Error: Please contact the administrator(3)");
     closeExit(1);
   }
 
@@ -179,18 +195,21 @@ int  main(int argc, char **argv)
   /* fork */
   if((pid=Fork())==-1){
     err_msg("ERR at %s#%d: fork error",__FILE__,__LINE__);
-    PutClientMsg("Error: Please contact to the administrator");
+    PutClientMsg("Error: Please contact the administrator(4)");
     closeExit(1);
   }
 
   if(pid!=0){
     /** parent process **/
-    /* send accept page with java */
-    PutClientAccept(userid, sessionId, port, pid, clientAddr4, clientAddr6, ipStatus, duration, durationEntered);
+    /* send accept page with javascript */
+    PutClientAccept(userid, sessionId, port, pid, clientAddr4, clientAddr6, 
+    		    ipStatus, duration, durationEntered, language,
+		    cookie, cookieAuth, redirectedUrl);
 
     /* detach from Web server */
     return 0;
   }
+
   /** child process **/
 
   /* detach from Web server */
@@ -200,28 +219,32 @@ int  main(int argc, char **argv)
 
   Pipe(dummyfd);      /* connect dummy pipe for stdin and out */
 
+  /* write the opening information to database */
+  /* this should be in child process as to save watch-process PID */
+  PutSessionBeginToDb(cookie, userid, clientAddr4, clientAddr6, 
+			macAddr4, ruleNumber4, ruleNumber6,
+		      duration, durationEntered, cookieAuth, language);
+
   /* get time at the client wait start (in msec) */
   gettimeofday(&timeBeginWait, NULL) ;
 
   /* wait connection from the client */
   /* if no connection, close gate when duration is passed */
   /* or ipaddr for the macAddr4 is changed */
-  connectionMode=WaitClientConnect(userid, userProperty, sessionId, clientAddr4, clientAddr6, duration, macAddr4, macAddr6, ipStatus, pClientAddr, language, port, parentpid);
+  connectionMode=WaitClientConnect(userid, userProperty, sessionId, 
+				   clientAddr4, clientAddr6, duration, 
+				   macAddr4, macAddr6, ipStatus, 
+				   pClientAddr, language, port, parentpid, 
+				   cookieAuth,redirectedUrl);
 
   /* get time at the client connection (in msec) */
   gettimeofday(&timeConnect, NULL) ;
 
-  if(connectionMode==JAVACONNECT){
-
-    /* wait until the java applet quit */
-    /* close gate when no reply to temporal hello or end of TCP connection */
-    /* macAddr6 is used for NDP to search addition ipaddr */
-    WaitJavaClose(pClientAddr, userid, userProperty, macAddr4, macAddr6, ipStatus);
-  }
-  else if(connectionMode==HTTPCONNECT){
+  if(connectionMode==HTTPCONNECT){
 
     /* wait until the http keep alive is closed */
-    WaitHttpClose(pClientAddr, userid, userProperty, macAddr4, macAddr6, ipStatus, sessionId, port);
+    WaitHttpClose(pClientAddr, userid, userProperty, macAddr4, macAddr6, 
+		  ipStatus, sessionId, port);
   }
 
   /* close firewall and exit */
@@ -256,6 +279,9 @@ void closeExit(int signo)
   /* save the connect mode */
   logConnectMode();
 
+  /* write closing information to database */
+  PutSessionEndToDb(cookie, mode[connectionMode]);
+
   /* signal is disabled */
   Signal(SIGTERM, SIG_DFL);
 
@@ -309,12 +335,11 @@ void setProcessTitle(char *useridshort, char *clientAddr4, char * ruleNumber4, c
 /*****************************************/
 void logConnectMode()
 {
-  char *mode[4]={"NONE","JAVA","HTTP","TIME"};
   long time1sec,time2sec,time3sec;
   long time1usec,time2usec,time3usec;
 
   /* set value on failure */
-  if(connectionMode<0 || connectionMode>3) connectionMode=NOCONNECT;
+  if(connectionMode<0 || connectionMode>2) connectionMode=NOCONNECT;
   if(connectionMode==NOCONNECT){
     gettimeofday(&timeBeginWait, NULL) ;
     gettimeofday(&timeConnect, NULL) ;
