@@ -62,6 +62,7 @@ typedef	void	Sigfunc(int);	/* for signal handlers */
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#include <openssl/md5.h>
 
 /***************** constants ***********************/
 
@@ -71,20 +72,22 @@ typedef	void	Sigfunc(int);	/* for signal handlers */
 #define PAMSERVICENAME "opengate"    /* default service name used in PAM */
 #define RADIUSCONF  "/etc/radius.conf" /* default path to radius.conf */
 
+#define COOKIENAME "OpengateAuth"  /* http-cookie name */
+
 #define ADDRMAXLN 128      /* maximum address string length */
 #define PASSMAXLN 128      /* maximum password string length */
 #define USERMAXLN 64       /* maximum userid string length */
 #define BUFFMAXLN 1024      /* maximum buffer string length */
 #define WORDMAXLN 64       /* maximum word length */
+#define SIDMAXLN 64       /* maximum sessionID length */
 #define TRUE 1
 #define FALSE 0
 #define DENY   0
 #define ACCEPT 1
 
 #define NOCONNECT 0    /* the client is not connected yet. */
-#define JAVACONNECT 1  /* the client is connected by using Java Applet */
-#define HTTPCONNECT 2  /* the client is connected by HTTP Keep-Alive */
-#define ENDCONNECT 3   /* the client is now terminating */
+#define HTTPCONNECT 1  /* the client is connected by HTTP Keep-Alive */
+#define ENDCONNECT 2   /* the client is now terminating */
 
 #define IPV4ONLY 4
 #define IPV46DUAL 46
@@ -138,20 +141,32 @@ struct clientAddr *CreateAddrListItem(char* ipAddr, char *ruleNumber, int ipType
 
 /* cgi interface */
 void GetClientAddr(char *clientAddr);
-int GetPostData(char *userid, char *password, char *clientAddr4, int *durationPtr, int *durationEntered);
-int GetUserProperty(char *userid, char *userProperty);
+int GetPostData(char *userid, char *password, char *clientAddr4, int *durationPtr, int *durationEntered,char *language, char *redirectedUrl);
+int GetCookieData(char *userid, char *clientAddr4, int *durationPtr, int *durationEntered,char *language);
+int GetAuthCookie(char *cookie, char *userid);
 void SplitId(char* userid, char* useridshort, char* extraId);
 int CheckReferer(void);
-void PutClientDeny(char *clientAddr4);
+void PutClientDeny(char *clientAddr4, char *language);
 void PutClientRetry(char *lang);
 void PutClientMsg(char *message);
-void PutClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered);
+void PutClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered, char *language, char *cookie, int cookieAuth, char *redirectedUrl);
+void split(char content[], char *name[], char *value[], char *next[]);
 
+/* db interface*/
+int PutSessionBeginToDb(char* cookie, char* userid, 
+			char* clientAddr4, char* clientAddr6, 
+			char* macAddr4, 
+			char* ruleNumber4, char* ruleNumber6,
+			int duration, int durationEntered, int cookieAuth, 
+			char *language);
+int PutSessionEndToDb(char* cookie, char* watchMode);
+int GetSessionInfoFromDb(char* cookie, char* userid, char* clientAddr4,
+			  char *macAddr, int *duration, int *durationEntered, char *language);
+int GetUserProperty(char *userid, char *userProperty);
 
 /* TCP communication with client */
-void WaitJavaClose(struct clientAddr *pClientAddr, char *userid, char *userProperty, char *macAddr4, char *macAddr6, int ipStatus);
 int GetListenPort(void);
-int WaitClientConnect(char *userid, char *userProperty, char *sessionId, char *clientAddr4, char *clientAddr6, int duration, char *macAddr4, char *macAddr6, int ipStatus, struct clientAddr *pClientAddr, char *language, int port, int pid);
+int WaitClientConnect(char *userid, char *userProperty, char *sessionId, char *clientAddr4, char *clientAddr6, int duration, char *macAddr4, char *macAddr6, int ipStatus, struct clientAddr *pClientAddr, char *language, int port, int pid, int cookieAuth, char *redirectedUrl);
 void WaitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userProperty, char *macAddr4, char *macAddr6, int ipStatus, char *sessionid, int port);
 
 /* utilities */
@@ -167,6 +182,7 @@ int Pclose(FILE *stream);
 int isNull(const char *pStr);
 char *GetServicePortStr(char *servName);
 void CreateSessionId(char *sessionId);
+void CreateCookie(char *cookie);
 char *md5hex(char *hexdigest, int len, char *str);
 
 void ReFormatMacAddr(char* macAddr4, char* macAddr6);
