@@ -319,8 +319,9 @@ char *getServicePortStr(char *servName)
 void createSessionId(char *sessionId)
 {
   srandom(getpid()+time(NULL));
-  snprintf(sessionId, BUFFMAXLN, "%ld", random() );
+  snprintf(sessionId, SIDMAXLN, "%ld", random() );
 }
+
 /*************************************************/
 /* calc MD5 in hex form                          */
 /*  str: plain text to convert                   */
@@ -341,7 +342,7 @@ char *md5hex(char *hexdigest, int len, char *str)
   }
 
   /* calc MD5 digest */
-  MD5(str, strlen(str), digest);
+  MD5((unsigned char*)str, strlen(str), digest);
 
   /* convert to HEX string */
   for(i=0;i<16;i++){
@@ -353,6 +354,18 @@ char *md5hex(char *hexdigest, int len, char *str)
   return hexdigest;
 }
 
+/*******************************************/
+/* create random session cookie            */
+/*******************************************/
+void createCookie(char *cookie)
+{
+  char str[BUFFMAXLN];
+
+  /* make Http-cookie from pid&time */
+  snprintf(str, BUFFMAXLN, "%d%d", getpid(),time(NULL));
+  md5hex(cookie, SIDMAXLN, str);
+}
+
 /****************************************/
 /****************************************/
 int Pclose(FILE *stream)
@@ -419,3 +432,8 @@ void CreateSessionId(char *sessionId){
   createSessionId(sessionId);
   if(debug>1) err_msg("DEBUG:<=createSessionId(%s)",sessionId);
 }
+void CreateCookie(char *cookie){
+  if(debug>1) err_msg("DEBUG:=>createCookie( )");
+  createCookie(cookie);
+  if(debug>1) err_msg("DEBUG:<=createCookie(%s)",cookie);
+}
