@@ -1,6 +1,6 @@
 /*************************************************
 Opengate server
-  module for communication with client program (java/javascript) 
+  module for communication with client program (javascript) 
 
 Copyright (C) 1999 Opengate Project Team
 Written by Yoshiaki Watanabe 1999-2006
@@ -29,7 +29,7 @@ void GetPeerAddr(int sockfd, char *peerAddr);
 void SendTerminateReply(void);
 void ReadHttpHeaders(void);
 void SendReplyToGetHello(void);
-void SendHttpKeepPage(char *userid, char *sessionId, char *language, int port);
+void SendHttpKeepPage(char *userid, char *sessionId, char *language, int port, int cookieAuth, char *redirectedUrl);
 int SelectAccept(void);
 void AcceptHttpReConnect(void);
 int MacAddrCheck(int ipStatus,char *clientAddr4, char *clientAddr6, char *macAddr4, char *macAddr6);
@@ -39,7 +39,6 @@ void SendHttpNotFound(void);
 
 void OnUsageTimeLimitAlarm(int signo);
 void OnCheckBasicAlarm(int signo);
-void OnCheckJavaAlarm(int signo);
 void OnCheckHttpAlarm(int signo);
 void OnReadWaitAlarm(int signo);
 void OnAjaxWaitAlarm(int signo);
@@ -143,12 +142,11 @@ int getListenPort(void)
 /************************************/
 /* wait for connection of client side program */
 /************************************/
-int waitClientConnect(char *userid, char *userProperty, char *sessionId, char *clientAddr4, char *clientAddr6, int duration, char *macAddr4, char *macAddr6, int ipStatus, struct clientAddr *pClientAddr, char *language, int port, int pid)
+int waitClientConnect(char *userid, char *userProperty, char *sessionId, char *clientAddr4, char *clientAddr6, int duration, char *macAddr4, char *macAddr6, int ipStatus, struct clientAddr *pClientAddr, char *language, int port, int pid, int cookieAuth, char *redirectedUrl)
 {
   char buff[BUFFMAXLN];             /* read in buffer */
   char connectAddr[ADDRMAXLN];      /* connected client address */
   char httpStr[BUFFMAXLN];          /* HTTP GET string at terminate */
-  char useridAndSessionId[BUFFMAXLN]; /* comcat userid and sessionid */
   int timeSendHttpkeep=0;
   int timeStart;
 
@@ -213,22 +211,6 @@ int waitClientConnect(char *userid, char *userProperty, char *sessionId, char *c
     }
     RemoveAlarm("ReadWaitAlarm");
     
-    /* is it the correct request from java applet */
-    /* the request is [<userid>:<sessionId>] */
-    snprintf(useridAndSessionId, BUFFMAXLN, "%s-%s", userid,sessionId);
-    if(strcmp(buff, useridAndSessionId)==0){
-
-      /* if too late request, ingore */
-      if((time(NULL)-timeStart) > atoi(GetConfValue("JavaWaitTimeout"))){
-	Close(connfd);
-	continue;
-      }
-
-      /* enter to the Java watch mode on the connection keeped */
-      connectMode=JAVACONNECT;
-      break;
-    }
-
     /* is it the httpkeep page download request */
     /* the request is [GET /httpkeep.html ....] */
     snprintf(httpStr, BUFFMAXLN, "GET /httpkeep-%s-%s", userid,sessionId);
@@ -236,7 +218,8 @@ int waitClientConnect(char *userid, char *userProperty, char *sessionId, char *c
       
       /* read out the remained headers and send the page */
       ReadHttpHeaders();
-      SendHttpKeepPage(userid, sessionId, language, port);
+      SendHttpKeepPage(userid, sessionId, language, port, cookieAuth, 
+		       redirectedUrl);
       timeSendHttpkeep=time(NULL);
 
       /* return to wait next request (that night be GET /hello) */
@@ -351,7 +334,7 @@ int selectAccept(void)
 void onUsageTimeLimitAlarm(int signo)
 {
   connectMode=ENDCONNECT;
-  err_msg("ERR at %s#%d: duration time out",__FILE__,__LINE__);
+  err_msg("ERR at %s#%d: duration timeout",__FILE__,__LINE__);
 }
 
 /***************************************/
@@ -462,118 +445,6 @@ void getPeerAddr(int sockfd, char *peerAddr)
   return;
 }
 
-/***************************************************/
-/* wait for close connection of java               */
-/***************************************************/
-void waitJavaClose(struct clientAddr *pClientAddr, char *userid, char *userProperty, char *macAddr4, char *macAddr6, int ipStatus)
-{
-  char buff[BUFFMAXLN];
-  int ret;
-
-  /* set alarm function arguments */
-  alarmArg.pClientAddr=pClientAddr;
-  alarmArg.macAddr4=macAddr4;
-  alarmArg.macAddr6=macAddr6;
-  alarmArg.userid=userid;
-  alarmArg.userProperty=userProperty;
-  alarmArg.ipStatus=ipStatus;
-
-  Writefmt(connfd,"accept\r\n");
-  
-  /* TCP read/write loop */
-  /* this loop implement following logic */
-  /*   repeat until receiving 'quit' or EOF from client         */
-  /*      say 'hello' to client                                 */
-  /*      receive 'hello' from client. if no reply, then quit.  */
-  /*      get packet count for client after short delay.        */
-  /*      long interval                                         */
-  /*      get packet count again. if no packet, then quit.      */
-  
-  /* set the alarm for periodic check */
-  AddAlarm("CheckJavaAlarm", alarmArg.checkInterval, FALSE, OnCheckJavaAlarm);
-  EnableAlarm();
-  
-  while(1){
-    /* set default to terminate */
-    connectMode=ENDCONNECT;
-    ret=readln(connfd, buff, BUFFMAXLN);
-
-    /* stop alarm */
-    DisableAlarm();
-
-    if(ret>=0){
-      /* normal read */
-      connectMode=JAVACONNECT;
-      /* read quit, then quit */
-      if(strstr(buff,"quit") != NULL){
-	break;
-      }
-      
-      /* read hello reply to the hello message in periodic check */
-      /* see onCheckJavaAlarm for timeout alarm setting */
-      else if((helloWait)&&(strstr(buff,"hello") !=NULL)){
-        helloWait=FALSE;
-	RemoveAlarm("ReplyTimeoutAlarm");
-      }
-      /* read other string */
-      else{
-      }
-    }
-
-    /* connection terminated */
-    if(connectMode==ENDCONNECT) break;
-
-    /* restart alarm */
-    EnableAlarm();
-  }
-
-  /* reset alarm */
-  RemoveAlarm(NULL);
-    
-  SendQuitClient();
-  
-  return;
-  }
-
-
-/***************************************/
-/* called at periodic java alive check */
-/***************************************/
-void onReplyTimeoutAlarm(int signo)
-{
-  /* no reply */
-  err_msg("ERR at %s#%d: no reply to hello",__FILE__,__LINE__);
-  connectMode=ENDCONNECT;
-}
-	
-/***************************************/
-/* called at periodic java alive check */
-/***************************************/
-void onCheckJavaAlarm(int signo)
-{
-  /* search new IPv6 addresses */
-  ScanNdpEntry(alarmArg.pClientAddr, alarmArg.userid,
-	     alarmArg.macAddr6, alarmArg.userProperty);
-
-  /* send hello and wait the reply */
-  Writefmt(connfd,"hello\r\n");
-  helloWait=TRUE;
-
-  /* mac address check */
-  if(MacAddrCheck(alarmArg.ipStatus, 
-		  alarmArg.clientAddr4, alarmArg.clientAddr6, 
-		  alarmArg.macAddr4, alarmArg.macAddr6)==FALSE){
-    connectMode=ENDCONNECT;
-    return;
-  }
-
-  /* set the alarm for next periodic check */
-  AddAlarm("CheckJavaAlarm", alarmArg.checkInterval, FALSE, OnCheckJavaAlarm);
-  /* EnableAlarm is called automatically in alarm function */
-
-  connectMode=JAVACONNECT;
-  return;
-}
 
 /***************************************************/
 /* send reply to terminate access via connfd(not stdout) */
@@ -670,8 +541,12 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
       else{
 	
 	/* read request and reply not found */
-	ReadHttpHeaders();
-	SendHttpNotFound();
+	if(strstr(buff,"GET /") == buff){
+	  ReadHttpHeaders();
+	  SendHttpNotFound();
+	}
+	connectMode=ENDCONNECT;
+	Close(connfd);
       }
     }
     else{
@@ -681,6 +556,9 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
 
       if(connectMode==ENDCONNECT){
 
+	/* close the connection */
+	Close(connfd);
+
 	/* wait short time to accept reconnection */
 	/* (to permit occasional disconnection at exchanging hello) */
 	if(helloWait==TRUE &&
@@ -724,11 +602,16 @@ void acceptHttpReConnect(void){
   /* connection wait */
   connfd = SelectAccept();
 
-  if(connfd<0) connectMode=ENDCONNECT; /* timeout or abnormal */
-  else         connectMode=HTTPCONNECT; /* reconnected */
-
   /* receive request */
   RemoveAlarm("ReadWaitAlarm");
+
+  if(connfd<0){  /* timeout or abnormal */
+    connectMode=ENDCONNECT;
+  }else{         /* reconnected */
+    connectMode=HTTPCONNECT;
+    RemoveAlarm("CheckHttpAlarm");
+    AddAlarm("CheckHttpAlarm", alarmArg.checkInterval, FALSE, OnCheckHttpAlarm);
+  }
 }
 
 /*****************************************************************/
@@ -868,7 +751,7 @@ void sendReplyToGetHello(void)
 /***************************************************/
 /* send httpkeep page to client via connfd         */
 /***************************************************/
-void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port)
+void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port, int cookieAuth, char *redirectedUrl)
 {
   char buff[BUFFMAXLN];             /* read in buffer */
   FILE *fp;
@@ -879,9 +762,9 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port)
   char acceptDoc2Url[BUFFMAXLN];
   char httpkeepJsUrl[BUFFMAXLN];
   char md5JsUrl[BUFFMAXLN];
-  char *startPageUrl=GetConfValue("StartPage/Url");
-  int startPageType=atoi(GetConfValue("StartPage/Type"));
   char portStr[WORDMAXLN];
+  char *startPageUrl="";
+  int startPageType=atoi(GetConfValue("StartPage/Type"));
 
   char *opengateServerName=GetConfValue("OpengateServerName");
   char *opengateDir=GetConfValue("OpengateDir");
@@ -924,6 +807,19 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port)
     return;
   }
 
+  /* if redirect page is not set, use other setting */
+  if(isNull(redirectedUrl)){
+    if(isNull(GetConfValue("StartPage/Url"))) redirectedUrl=acceptDoc2Url;
+    else  redirectedUrl=GetConfValue("StartPage/Url");
+  }
+
+  /* create start page url to put information */
+  if(cookieAuth==1) startPageUrl=redirectedUrl;
+  else if(startPageType==0) startPageUrl=acceptDoc2Url;
+  else if(startPageType==1) startPageUrl=GetConfValue("StartPage/Url");
+  else if(startPageType==2) startPageUrl=redirectedUrl;
+  else startPageUrl=acceptDoc2Url;
+
   /* send HTTP headers */
   Writefmt(connfd,"HTTP/1.1 200 OK\r\n");
   Writefmt(connfd,"Transfer-Encoding: chunked\r\n");
@@ -936,7 +832,8 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port)
 
     /* length check */
     if(strlen(buff)>=BUFFMAXLN-1){
-      err_msg("ERR at %s#%d: too long line in %s",__FILE__,__LINE__,httpKeepDoc);
+      err_msg("ERR at %s#%d: too long line in %s",__FILE__,__LINE__,
+	      httpKeepDoc);
     }
 
     /* replace mark */
@@ -951,13 +848,8 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port)
     htmlReplace(buff, "%%OPENGATEDIR%%", opengateDir);
     htmlReplace(buff, "%%OPENGATEPORT%%", portStr);
     htmlReplace(buff, "%%LANGUAGE%%", language);
-
-    /* replace start url mark */
-    if( startPageType==1 ){
-      htmlReplace(buff, "%%STARTURL%%", startPageUrl);
-    }else{
-      htmlReplace(buff, "%%STARTURL%%", acceptDoc2Url);
-    }
+    htmlReplace(buff, "%%STARTURL%%", startPageUrl);
+    htmlReplace(buff, "%%COOKIENAME%%", COOKIENAME);
 
     /* length of chunk in hex */
     Writefmt(connfd, "%x\r\n", strlen(buff));
@@ -1029,24 +921,17 @@ int GetListenPort(void)
   return ret;
 }
 
-int WaitClientConnect(char *userid, char *userProperty, char *sessionId, char *clientAddr4, char *clientAddr6, int duration, char *macAddr4, char *macAddr6, int ipStatus, struct clientAddr *pClientAddr, char *language, int port, int pid)
+int WaitClientConnect(char *userid, char *userProperty, char *sessionId, char *clientAddr4, char *clientAddr6, int duration, char *macAddr4, char *macAddr6, int ipStatus, struct clientAddr *pClientAddr, char *language, int port, int pid, int cookieAuth, char *redirectedUrl)
 {
   int ret;
 
-  if(debug>1) err_msg("DEBUG:=>waitClientConnect(%s,%s,%s,%s,%s,%d,%s,%s,%d,%s,%d,%d)",userid,userProperty,sessionId,clientAddr4,clientAddr6,duration,macAddr4,macAddr6,ipStatus,language,port,pid);
-  ret=waitClientConnect(userid,userProperty,sessionId,clientAddr4,clientAddr6,duration,macAddr4,macAddr6,ipStatus,pClientAddr,language,port,pid);
+  if(debug>1) err_msg("DEBUG:=>waitClientConnect(%s,%s,%s,%s,%s,%d,%s,%s,%d,%s,%d,%d,%d,%s)",userid,userProperty,sessionId,clientAddr4,clientAddr6,duration,macAddr4,macAddr6,ipStatus,language,port,pid,cookieAuth,redirectedUrl);
+  ret=waitClientConnect(userid,userProperty,sessionId,clientAddr4,clientAddr6,duration,macAddr4,macAddr6,ipStatus,pClientAddr,language,port,pid,cookieAuth,redirectedUrl);
   if(debug>1) err_msg("DEBUG:(%d)<=waitClientConnect( )",ret);
 
   return ret;
 }
 
-void WaitJavaClose(struct clientAddr *pClientAddr, char *userid, char *userProperty, char *macAddr4, char *macAddr6, int ipStatus)
-{
-  if(debug>1) err_msg("DEBUG:=>waitJavaClose(%p,%s,userProperty,%s,%s,%d)",pClientAddr,userid,macAddr4,macAddr6,ipStatus);
-  waitJavaClose(pClientAddr,userid,userProperty,macAddr4,macAddr6,ipStatus);
-  if(debug>1) err_msg("DEBUG:<=waitJavaClose( )");
-}
-
 void SendQuitClient(void)
 {
   if(debug>1) err_msg("DEBUG:=>sendQuitClient( )");
@@ -1083,10 +968,10 @@ void SendReplyToGetHello(void)
   if(debug>1) err_msg("DEBUG:<=sendReplyToGetHello( )");
 }
 
-void SendHttpKeepPage(char *userid, char *sessionId, char *language, int port)
+void SendHttpKeepPage(char *userid, char *sessionId, char *language, int port, int cookieAuth, char *redirectedUrl)
 {
-  if(debug>1) err_msg("DEBUG:=>sendHttpKeepPage(%s,%s,%s,%d)", userid, sessionId, language, port);
-  sendHttpKeepPage(userid, sessionId, language, port);
+  if(debug>1) err_msg("DEBUG:=>sendHttpKeepPage(%s,%s,%s,%d,%d,%s)", userid, sessionId, language, port, cookieAuth, redirectedUrl);
+  sendHttpKeepPage(userid, sessionId, language, port, cookieAuth, redirectedUrl);
   if(debug>1) err_msg("DEBUG:<=sendHttpKeepPage( )");
 
 }
@@ -1105,13 +990,6 @@ void OnCheckBasicAlarm(int signo){
   if(debug>1) err_msg("DEBUG:<=onCheckBasicAlarm()");
 }
 
-void OnCheckJavaAlarm(int signo){
-
-  if(debug>1) err_msg("DEBUG:=>onCheckJavaAlarm()");
-  onCheckJavaAlarm(signo);
-  if(debug>1) err_msg("DEBUG:<=onCheckJavaAlarm()");
-}
-
 void OnCheckHttpAlarm(int signo){
 
   if(debug>1) err_msg("DEBUG:=>onCheckHttpAlarm()");
@@ -1182,5 +1060,3 @@ void SendHttpNotFound(void){
   sendHttpNotFound();
   if(debug>1) err_msg("DEBUG:<=sendHttpNotFound()");
 }
-
-
